<!DOCTYPE html>
<html lang="en">
<head>
<link rel="stylesheet" type="text/css" href="/static/css/fonts.css">
<link href="/static/css/root.css" rel="stylesheet">
<link href="/static/css/base.css" rel="stylesheet">
<title>{{ title }}</title>

{% block header %}
{% endblock %}

</head>
<body>
    {% block main %}
    {% endblock %}
</body>

{% block script %}
{% endblock %}
</html>